#include "stdafx.h"
#include "NeMessenger.h"
#include "ChatSplitterWnd.h"

#define DEF_BG_COLOR				Color(255,230,233,238)
#define DEF_LINE1_COLOR				Color(255,238,238,238)
#define DEF_LINE2_COLOR				Color(255,185,185,193)
#define DEF_LINE3_COLOR				Color(255,239,242,247)
#define DEF_BG_DOWN_COLOR			Color(255,180,188,203)

IMPLEMENT_DYNAMIC(CChatSplitterWnd, CWnd)

CChatSplitterWnd::CChatSplitterWnd()
{
	// BG
	m_pImgBG = NULL;
	m_pImgSplitter = NULL;

	m_crBG = DEF_BG_COLOR;

	// dimension
	m_rtWnd.SetRectEmpty();
	m_rtSplitter.SetRectEmpty();

	// tracking
	m_bTracking = FALSE;
	m_bHoverSplitter = FALSE;
	m_bSplitterPressed = FALSE;

	m_pWndMoveUp = NULL;
	m_pWndMoveDown = NULL;

	m_ptMoveStart = m_ptMoveEnd = CPoint(0, 0);
}

CChatSplitterWnd::~CChatSplitterWnd()
{
    ASSERT(NULL == m_pImgBG);
    ASSERT(NULL == m_pImgSplitter);

    //fix lint 1540
    SAFE_DELETE(m_pImgBG);
    SAFE_DELETE(m_pImgSplitter);
    m_pWndMoveUp = NULL;
    m_pWndMoveDown = NULL;
}

BEGIN_MESSAGE_MAP(CChatSplitterWnd, CWnd)
	ON_WM_CREATE()
	ON_WM_ERASEBKGND()
	ON_WM_PAINT()
	ON_WM_SETCURSOR()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_SIZE()
	ON_WM_DESTROY()
	ON_WM_MOUSELEAVE()	
END_MESSAGE_MAP()

void CChatSplitterWnd::SetImgBGPath(CString strFileName)
{
	CString strImgPath = _T(""), strImgDir = _T("");

	strImgDir = GetAppImgDir();

	if (strFileName.IsEmpty())
		return;
	strImgPath.Format(_T("%s\\%s"), strImgDir, strFileName);

	m_pImgBG = new Image(strImgPath);
}

void CChatSplitterWnd::SetImgSplitterPath(CString strFileName)
{
	CString strImgPath = _T(""), strImgDir = _T("");

	strImgDir = GetAppImgDir();

	if (strFileName.IsEmpty())
		return;
	strImgPath.Format(_T("%s\\%s"), strImgDir, strFileName);

	m_pImgSplitter = new Image(strImgPath);
}

void CChatSplitterWnd::SetSplitterBuddy(CWnd* pWndUp, CWnd* pWndDown)
{
	m_pWndMoveUp = pWndUp;
	m_pWndMoveDown = pWndDown;
}

int CChatSplitterWnd::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	return 0;
}

void CChatSplitterWnd::OnDestroy()
{
	SAFE_DELETE(m_pImgBG);
	SAFE_DELETE(m_pImgSplitter);

	CWnd::OnDestroy();
}

BOOL CChatSplitterWnd::OnEraseBkgnd(CDC* pDC)
{
	return TRUE;
}

void CChatSplitterWnd::OnPaint()
{
	CPaintDC dc(this);
	
	PAINTSTRUCT ps = dc.m_ps;
	int psWidth = ps.rcPaint.right - ps.rcPaint.left;
	int psHeight = ps.rcPaint.bottom - ps.rcPaint.top;

	int nPX = ps.rcPaint.left;
	int nPY = ps.rcPaint.top;

	CRect rtPS(nPX, nPY, nPX + psWidth, nPY + psHeight);

    CRect rt(0, 0, 0, 0);
	GetClientRect(&rt);
	int nWndWidth = rt.Width();
	int nWndHeight = rt.Height();

	HDC hdc = dc.GetSafeHdc();

	HDC	hdcMemory = CreateCompatibleDC(hdc);

	if (hdcMemory == NULL)
		return;

	HBITMAP hbmpMem = CreateCompatibleBitmap(hdc, psWidth, psHeight);

	HGDIOBJ hbmpOld = ::SelectObject(hdcMemory,  hbmpMem);

	Graphics graph(hdcMemory);

	if (hbmpMem)
	{
		graph.SetPageUnit(UnitPixel);

		ImageAttributes ImgAttr;

		ImageAttributes ImgAttrHover;
		ColorMatrix matrixH = {	0.80f, 0.00f, 0.00f, 0.00f, 0.00f,
								0.00f, 0.80f, 0.00f, 0.00f, 0.00f,
								0.00f, 0.00f, 0.80f, 0.00f, 0.00f,
								0.00f, 0.00f, 0.00f, 1.00f, 0.00f,
								0.00f, 0.00f, 0.00f, 0.00f, 1.00f	};
		ImgAttrHover.SetColorMatrix(&matrixH);

		ImageAttributes ImgAttrDown;
		ColorMatrix matrixD = {	1.20f, 0.00f, 0.00f, 0.00f, 0.00f,
								0.00f, 1.20f, 0.00f, 0.00f, 0.00f,
								0.00f, 0.00f, 1.20f, 0.00f, 0.00f,
								0.00f, 0.00f, 0.00f, 1.00f, 0.00f,
								0.00f, 0.00f, 0.00f, 0.00f, 1.00f	};		
		ImgAttrDown.SetColorMatrix(&matrixD);

		if (m_pImgBG)
			DrawImageToDest(&graph, m_pImgBG, &rtPS, &m_rtWnd, &ImgAttr);
		else
		{
			SolidBrush brushBG(m_crBG);
			Pen penBG(DEF_LINE1_COLOR, 1);

/*
			if (m_bSplitterPressed)
			{
				brushBG.SetColor(DEF_BG_DOWN_COLOR);
				penBG.SetColor(DEF_BG_DOWN_COLOR);
			}
*/

			graph.FillRectangle(&brushBG, 0, 0, nWndWidth, nWndHeight);
			graph.DrawLine(&penBG, 0, nWndHeight - 1, nWndWidth, nWndHeight - 1);
			penBG.SetColor(DEF_LINE2_COLOR);
			graph.DrawLine(&penBG, 0, nWndHeight - 2, nWndWidth, nWndHeight - 2);
			penBG.SetColor(DEF_LINE3_COLOR);
			graph.DrawLine(&penBG, 0, nWndHeight - 3, nWndWidth, nWndHeight - 3);
		}

		if (m_pImgSplitter)
		{
			ImageAttributes* pImgAttr = &ImgAttr;
/*
			if (m_bHoverSplitter)
			{
				pImgAttr = &ImgAttrHover;
				if (m_bSplitterPressed)
					pImgAttr = &ImgAttrDown;
			}
*/
			DrawImageToDest(&graph, m_pImgSplitter, &rtPS, &m_rtSplitter, pImgAttr);
		}

		BitBlt(dc.GetSafeHdc(), nPX, nPY, psWidth, psHeight, hdcMemory, 0, 0, SRCCOPY);

		::SelectObject(hdcMemory, hbmpOld);
		::DeleteObject(hbmpMem);
	}

	::DeleteDC(hdcMemory);
}

BOOL CChatSplitterWnd::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message)
{
	::SetCursor(AfxGetApp()->LoadCursor(AFX_IDC_VSPLITBAR));

	return TRUE;
}

BOOL CChatSplitterWnd::DoSplitter(int nMove)
{
	if (nMove == 0)
		return FALSE;

	CString strMsg;
	BOOL bMove = TRUE;

	if (nMove > 0)
	{		
		if (m_pWndMoveDown && m_pWndMoveDown->GetSafeHwnd())
			bMove = m_pWndMoveDown->SendMessage(UM_SPLITTER_MOVE, (WPARAM)0, (LPARAM)nMove);

		if (bMove)
		{
			SplitterMove(nMove);

			if (m_pWndMoveUp && m_pWndMoveUp->GetSafeHwnd())
				m_pWndMoveUp->SendMessage(UM_SPLITTER_MOVE, (WPARAM)0, (LPARAM)nMove);
		}
	}
	else
	{
		if (m_pWndMoveUp && m_pWndMoveUp->GetSafeHwnd())
			bMove = m_pWndMoveUp->SendMessage(UM_SPLITTER_MOVE, (WPARAM)0, (LPARAM)nMove);

		if (bMove)
		{
			SplitterMove(nMove);

			if (m_pWndMoveDown && m_pWndMoveDown->GetSafeHwnd())
				m_pWndMoveDown->SendMessage(UM_SPLITTER_MOVE, (WPARAM)0, (LPARAM)nMove);
		}
	}

	return bMove;
}

void CChatSplitterWnd::SplitterMove(int nMove)
{
	CRect rtSplitter(0, 0, 0, 0);
	GetWindowRect(&rtSplitter);

	CWnd* pWndParent = GetParent();
	if (!pWndParent)
		return;
	pWndParent->ScreenToClient(&rtSplitter);

	rtSplitter.OffsetRect(0, nMove);
	MoveWindow(&rtSplitter);
}

void CChatSplitterWnd::OnMouseMove(UINT nFlags, CPoint point)
{
	m_bHoverSplitter = TRUE;

	if (m_bSplitterPressed)
	{
		GetCursorPos(&m_ptMoveEnd);
		CPoint ptDiff = m_ptMoveEnd - m_ptMoveStart;

		//Trace(_T("OnMouseMove(%d) %d\n"), m_ptMoveStart.y, m_ptMoveEnd.y);

		if (DoSplitter(ptDiff.y) == TRUE)
			m_ptMoveStart = m_ptMoveEnd;
	}

	if (!m_bTracking)
	{
		m_bTracking = TRUE;

		TRACKMOUSEEVENT tme;
		tme.cbSize = sizeof(TRACKMOUSEEVENT);
		tme.dwFlags = TME_HOVER | TME_LEAVE;
		tme.dwHoverTime = 1;
		tme.hwndTrack = GetSafeHwnd();
		TrackMouseEvent(&tme);

		RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE | RDW_ALLCHILDREN);
	}

	CWnd::OnMouseMove(nFlags, point);
}

void CChatSplitterWnd::OnMouseLeave()
{
	m_bTracking = FALSE;
	m_bHoverSplitter = FALSE;

	if (GetAsyncKeyState(VK_LBUTTON) & 0x8000)
	{
	}
	else if (m_bSplitterPressed)
	{
		ReleaseCapture();

		if (m_bSplitterPressed)
		{
			GetCursorPos(&m_ptMoveEnd);
			CPoint ptDiff = m_ptMoveEnd - m_ptMoveStart;

			DoSplitter(ptDiff.y);

			m_ptMoveStart = m_ptMoveEnd;
		}

		m_bSplitterPressed = FALSE;
		m_ptMoveStart = m_ptMoveEnd = CPoint(0, 0);
	}

	RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE | RDW_ALLCHILDREN);
}

void CChatSplitterWnd::OnLButtonDown(UINT nFlags, CPoint point)
{
	SetFocus();

	m_bSplitterPressed = TRUE;
	GetCursorPos(&m_ptMoveStart);

	RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE | RDW_ALLCHILDREN);

	SetCapture();

	CWnd::OnLButtonDown(nFlags, point);
}

void CChatSplitterWnd::OnLButtonUp(UINT nFlags, CPoint point)
{
	ReleaseCapture();

	if (m_bSplitterPressed)
	{
		GetCursorPos(&m_ptMoveEnd);
		CPoint ptDiff = m_ptMoveEnd - m_ptMoveStart;

		DoSplitter(ptDiff.y);

		m_ptMoveStart = m_ptMoveEnd;
	}

	m_bSplitterPressed = FALSE;
	m_ptMoveStart = m_ptMoveEnd = CPoint(0, 0);

	RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE | RDW_ALLCHILDREN);

	CWnd::OnLButtonUp(nFlags, point);
}

void CChatSplitterWnd::OnSize(UINT nType, int cx, int cy)
{
	CWnd::OnSize(nType, cx, cy);

	m_rtWnd.SetRect(0, 0, cx, cy);

	if (m_pImgSplitter)
	{
		int nImgWidth = m_pImgSplitter->GetWidth();
		int nImgHeight = m_pImgSplitter->GetHeight();
		int nSX = (cx - nImgWidth) / 2;
		int nSY = 0;
		int nCX = nImgWidth;
		int nCY = nImgHeight;
		m_rtSplitter.SetRect(nSX, nSY, nSX + nCX, nSY + nCY);
	}
}
